##
## Module to fetch_stat.pl from the Dell PowerConnect 2708, 2716 and 2724
## To handle the 2748 use the module dell_2748.pm 
##
## Written by Joakim Andersson, joakim at iqcc dot se
##

use strict;
use LWP::UserAgent;
use Net::HTTP;
use Digest::MD5  qw(md5_hex);

our $modver  = '1.02 (2009-04-26)';
our $modname = "dell_27xx.pm";
our $modcreator = "Joakim Andersson";

#######################################################
# Shared variables

our $SET;			# Settings
our $SYSTEM;		# System-info

# Creating UserAgent-object
my $ua = LWP::UserAgent->new();
$ua->agent("Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");

##
## Poll port
##

sub switch_poll_port
{
	my $portnum = shift;
	my $is_first_poll = shift || 0;
	
	my $logtag = "$modname: poll_port:";
	
	##
	## Is it a port or LAG?
	##
	
	my $arg = ($portnum =~ /^T(\d+)$/) ? "TrunkNo=$1" : "PortNo=$portnum";
		
	# Fetches the page
	my $res = $ua->get("http://$SET->{'SWITCH'}/portStats.htm?$arg",
	       'Cookie' => $SYSTEM->{'cookie'}
		 );
	
	&log_n_die("$logtag Unable to fetch /portStats.htm?$arg - ".$res->status_line) unless ($res->is_success);
	
	# Parsing content
	my $data = $res->content;
	$data =~ tr/\r//d;
	
	if ($data =~ /^<title>Login<\/title>$/m)
	{
		##
		## Invalid session!
		##
		
		&log("Session invalid", $SET->{'VERBOSE'});
		
		$SYSTEM->{'cookie'} = '';
		return 0;
	}
	
	##
	## Parses the data, is stored in javascript-variables in the beginning of the page
	##
	
	my $row = 0;
	
	my $name = '';
	my %vars = ();
	
	foreach (split(/\n+/, $data))
	{
		$row++;
		#print "RAD $row: $_\n";
		
		last if (/^<\/script>$/);	# May need adjustment for future firmwares
		next unless (/^var (.+)="(\d*)";$/);
		
		my $name = $1;
		next if ($vars{$name});		# Skips already found variables, a bug in the current firmware
		
		my $value = reverse $2;		# Values are stored reversed in the HTML-page
		$value += 0;				# Perhaps unneeded, but zero is better that nothing
		
		$vars{$name} = $value;
	}
	
	return \%vars;
}

##
## Resets stats for all ports
##

sub switch_clear_stat
{
	if ($SYSTEM->{'numberOfPorts'} > 0)
	{
		my $ok = 0;
		foreach my $num (1 .. $SYSTEM->{'numberOfPorts'})
		{
			my $sock = Net::HTTP->new(Host => $SET->{'SWITCH'}) || die $@;
			$sock->write_request(POST => "/tgi/portstats.tgi",
				'User-Agent' => "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)",
				'Content-Type' => 'application/x-www-form-urlencoded',
				'Cache-Control' => 'no-cache',
				'Accept' => '*/*',
				'Cookie' => $SYSTEM->{'cookie'},
				'Referer' => "http://$SET->{'SWITCH'}/portStats.htm",
				"PortNo=$num&PortNo\$select=$num&RefreshRate\$select=15000&clrCnts=OK"
				);
			
			my ($code, $mess, %h) = $sock->read_response_headers;
			$ok += ($code == 302);		# 302 = Probably OK
		}
		print "Cleared $ok ports\n";
	}
	if ($SYSTEM->{'numberOfTrunks'} > 0)
	{
		my $ok = 0;
		foreach my $num (1 .. $SYSTEM->{'numberOfTrunks'})
		{
			my $sock = Net::HTTP->new(Host => $SET->{'SWITCH'}) || die $@;
			$sock->write_request(POST => "/tgi/portstats.tgi",
				'User-Agent' => "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)",
				'Content-Type' => 'application/x-www-form-urlencoded',
				'Cache-Control' => 'no-cache',
				'Accept' => '*/*',
				'Cookie' => $SYSTEM->{'cookie'},
				'Referer' => "http://$SET->{'SWITCH'}/portStats.htm",
				"TrunkNo=$num&TrunkNo\$select=$num&RefreshRate\$select=15000&clrCnts=OK"
				);
			
			my ($code, $mess, %h) = $sock->read_response_headers;
			$ok += ($code == 302);		# 302 = Probably OK
		}
		print "Cleared $ok LAGs\n";
	}
	
	return 1;
}

##
## Reading systeminfo that doesn't change (often)
##

sub switch_system_info
{
	my $logtag = "$modname: system_info:";
	
	my $res = $ua->get("http://$SET->{'SWITCH'}/portConfig.htm",
	       'Cookie' => $SYSTEM->{'cookie'}
		 );
	
	&log_n_die("$logtag Unable to fetch /portConfig.htm - ".$res->status_line) unless ($res->is_success);
	
	# Content
	my $data = $res->content;
	$data =~ tr/\r//d;
	
	if ($data =~ /^<title>Login<\/title>$/m)
	{
		##
		## Invalid session!
		##
		
		&log("Session invalid", $SET->{'VERBOSE'});
		
		$SYSTEM->{'cookie'} = '';
		return 0;
	}
	
	##
	## Finds the info we needs
	##
	
	my $row = 0;
	my %vars = ();
	foreach (split(/\n+/, $data))
	{
		$row++;
		#print "RAD $row: $_\n";
		
		last if (/^<\/script>$/);	# May need adjustment in future firmwares
		next unless (/^var (.+)="(\d*)";$/);
		
		my $name = $1;
		my $value = $2;
		
		#next if ($stat{$name});
		#print "  $name -> $value\n";
		$vars{$name} = $value;
	}
	
	$SYSTEM->{'numberOfPorts'} = $vars{'numberOfPorts'};
	$SYSTEM->{'numberOfTrunks'} = $vars{'numberOfTrunks'};
	
	##
	## Parses info about every port
	##
	
	my $tot = $vars{'numberOfPorts'} + $vars{'numberOfTrunks'};
	my %ports = ();
	
	foreach my $num (1 .. $tot)
	{
		my $trunknum = $num - $vars{'numberOfPorts'};
		my $portnum = $num;
		$portnum = "T$trunknum" if ($trunknum > 0);	# It's a trunk
		
		# var portLinkList="0101110101011100000000";
		# var portSpeedList="0202110201012200333333";
		# var portDuplexList="0101110101011100222222";
		# var portANList="1111111111111111222222";
		# var portBPList="0000000000000000222222";
		# var portFcList="0100000100000000333333";
		
		my $index = $num - 1;
		my $portLink   = substr($vars{'portLinkList'},   $index, 1);
		my $portSpeed  = substr($vars{'portSpeedList'},  $index, 1);
		my $portDuplex = substr($vars{'portDuplexList'}, $index, 1);
		my $portFc     = substr($vars{'portFcList'},     $index, 1);
		my $portBP     = substr($vars{'portBPList'},     $index, 1);
		
		#$PORT->{'link'} = '-' if ($PORT->{'link'} eq 'up?');
		my $link = 'down';
		
		if ($portLink)
		{
			$link = ('10','100','1000','active')[$portSpeed];
			$link .= ('/Half','/Full','')[$portDuplex];
			$link .= '/FC' if ($portFc == 1);
			$link .= '/BP' if ($portBP == 1);
		}
		
		$ports{$portnum} = {};
		$ports{$portnum}->{'link'} = $link;
		$ports{$portnum}->{'is_active'} = $portLink;
		$ports{$portnum}->{'order'} = $num;
	}
	
	##
	## Fetching system info
	##
	
	$res = $ua->get("http://$SET->{'SWITCH'}/system.htm",
	       'Cookie' => $SYSTEM->{'cookie'}
		 );
	
	&log_n_die("$logtag: Unable to fetch /system.htm - ".$res->status_line) unless ($res->is_success);
	
	# Parsing content
	$data = $res->content;
	$data =~ tr/\r//d;
	
	if ($data =~ /^<title>Login<\/title>$/m)
	{
		##
		## Invalid session!
		##
		
		&log("Session invalid", $SET->{'VERBOSE'});
		
		$SYSTEM->{'cookie'} = '';
		return 0;
	}
	
	##
	## Parses the data, is stored in javascript-variables in the beginning of the page
	##
	
	# var productName="PowerConnect 2716";
	# var macAddr="00:13:72:F5:68:68";
	# var assetTag="";
	# var serviceTag="CCZR291";
	# var systemName="";
	# var locationName="";
	# var sysUpTime="0005597773";
	# var serialNum="CN0W88752829863M0026";
	# var fwVer="1.0.1.07";
	# var bootVer="1008";
	
	%vars = ();
	$row = 0;
	foreach (split(/\n+/, $data))
	{
		$row++;
		
		last if (/^<\/script>$/);	# May need adjustment for future firmwares
		next unless (/^var (.+)="(.*)";$/);
		
		$vars{$1} = $2;
	}
	
	# Populating SYSTEM with the vars
	$SYSTEM->{'model'}      = ($vars{'productName'}) ? "Dell $vars{'productName'}" : '';
	$SYSTEM->{'macaddress'} = $vars{'macAddr'};
	$SYSTEM->{'servicetag'} = $vars{'serviceTag'};
	$SYSTEM->{'assettag'}  	= $vars{'assetTag'};
	$SYSTEM->{'sysname'}    = $vars{'systemName'};
	$SYSTEM->{'location'}   = $vars{'locationName'};
	$SYSTEM->{'serial'}     = $vars{'serialNum'};
	
	# var fwVer="1.0.1.07";
	# var bootVer="1008";
	$SYSTEM->{'firmware'}   = "$vars{'fwVer'}/$vars{'bootVer'}";
	
	# var sysUpTime="0005597773";
	# Strips the meaningless leading zeros and the millisec
	my $secs = $vars{'sysUpTime'};
	$secs =~ s/\d\d$//;	# Strips last two digits to only have seconds left
	$secs += 0;
	$SYSTEM->{'uptime'} = $secs;
	
	return \%ports;
}

##
## Poll MACs
##

sub switch_poll_macs
{
	my $logtag = "$modname: poll_macs:";
	
	return 0;
}	

##
## Login to setup the session
##

sub switch_login
{
	my $logtag = "$modname: login:";
	
	my $res = $ua->get("http://$SET->{'SWITCH'}/login11.htm");
	&log_n_die("Unable to fetch /login11.htm - ".$res->status_line) unless ($res->is_success);
	
	my $data = $res->content;
	my $session = '';
	
	# Finds the Session-value, needed to compute the password that is sent when logging in
	if ($data =~ /^<input type="hidden" name="Session" value="(.*)">/m)
	{
		$session = $1;
	}
	
	unless ($session)
	{
		&log_n_die("$logtag Unable to find variable Session in the page! $data");
	}
	
	if ($session =~ /^0+$/)
	{
		&log_n_die("$logtag Session $session is invalid! You may need to wait a while, or possibly reboot the switch");
	}
	
	print "Found 'Session': $session\n" if ($SET->{'VERBOSE'} >= 3);
	#$session = '8c6e05969fb0748f7fa239628432ffe4';
	
	##
	## Logging in to fetch the cookie
	##
	
	my $md5_password = md5_hex($SET->{'USERNAME'} . $SET->{'PASSWORD'} . $session);
	print "Computed MD5: $md5_password\n" if ($SET->{'VERBOSE'} >= 3);
	
	my $sock = Net::HTTP->new(Host => $SET->{'SWITCH'}) || die $@;
	$sock->write_request(POST => "/tgi/login.tgi",
					'User-Agent' => "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)",
					'Content-Type' => 'application/x-www-form-urlencoded',
					'Cache-Control' => 'no-cache',
					'Accept' => '*/*',
					"Username=$SET->{'USERNAME'}&Password=$md5_password&Session=$session"
			);
	
	my($code, $mess, %h) = $sock->read_response_headers;
	
	unless ($code == 302)
	{
		&log_n_die("$logtag Didn't recieve code 302 when logging in! $mess");
	}
	
	##
	## Finding the Cookie we should recieve when logged in successfully
	##
	## Set-Cookie : SSID=0f9a03b3d15acd65af5dfcf6550c5eb6; path=/
	##
	
	unless ($h{'Set-Cookie'})
	{
		foreach (sort keys %h)
		{
			print "$_ = $h{$_}\n";
		}
			
		&log_n_die("$logtag Didn't recieve cookie! $h{'Set-Cookie'}");
	}
	
	$h{'Set-Cookie'} =~ /(SSID=.+?)\;/ || &log_n_die("$logtag Unable to interpret SSID from $h{'Set-Cookie'}!");
	return $1;
}

##
## Logout
##

sub switch_logout
{
	my $logtag = "$modname: logout";
	
	# /logOffPage.htm
	my $res = $ua->get("http://$SET->{'SWITCH'}/logoff.htm",
	       'Cookie' => $SYSTEM->{'cookie'}
		 );
	
	&log_n_die("$logtag: Unable to fetch /logoff.htm - ".$res->status_line) unless ($res->is_success);
	my $data = $res->content;
	
	#print $data;
	
	# Assume it worked
	$SYSTEM->{'cookie'} = '';
	
	return 1;
}

###########################################################

&log("$modname $modver loaded", $SET->{'VERBOSE'});

1;

