##
## Module to fetch_stat.pl from the HP 1800-24G/8G
## 
## Written by Joakim Andersson, joakim at iqcc dot se
##

use strict;
use LWP::UserAgent;
use Net::HTTP;
use HTTP::Message;

our $modver  = '1.00 (2009-10-21)';
our $modname = "hp_1800g.pm";
our $modcreator = "Joakim Andersson";

#######################################################
# Shared variables

our $SET;			# Settings
our $SYSTEM;		# System-info

# Creating UserAgent-object
my $ua = LWP::UserAgent->new();
$ua->agent("Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");
my $accept_encoding = HTTP::Message::decodable;

##
## Poll port
##

my %poll_ports_cache = ();

sub switch_poll_port
{
	my $portnum = shift;
	my $is_first_poll = shift || 0;
	
	my $logtag = "$modname: poll_port:";
	
	##
	## Is it a port or LAG?
	##
	
	$portnum =~ /^(T)?(\d+)$/ || &log_n_die("$logtag Invalid portnum $portnum!");
	my $is_trunk = $1;
	my $id = $2 - 1;
	
	if ($is_trunk)
	{
		&log("Sorry, don't know yet how to poll a trunk :-(", 1);
		return {};
	}
	
	if ($is_first_poll)
	{
		##
		## We get all ports on the same time, so we cache it
		##
		
		#my $res = $ua->post("http://$SET->{'SWITCH'}/support/loadsupport.html", {'R10' => '2', 'submit' => 'APPLY'}, 'Accept-Encoding' => $accept_encoding);
		my $res = $ua->get("http://$SET->{'SWITCH'}/support/loadsupport.html?R10=2&submit=APPLY", 'Accept-Encoding' => $accept_encoding);
		&log_n_die("$logtag Unable to fetch /support/loadsupport.html - ".$res->status_line) unless ($res->is_success);
		
		my $data = $res->decoded_content;
		my $title = '';
		$title = $1 if ($data =~ /<title>([^<]+)<\/title>/);
		
		if ($data =~ /Forbidden Access/ || $title =~ /(Duplicate Administrator|Login)/)
		{
			print $data;
			
			&log("Session invalid", $SET->{'VERBOSE'});
			exit;
			
			$SYSTEM->{'cookie'} = '';
			return 0;
		}
		
		&log("Fetched /support/loadsupport.html, got title $title", $SET->{'VERBOSE'} >= 3);
		
		# Port  1:
		# Rx Packets:                      0 Tx Packets:                      0 
		# Rx Octets:                       0 Tx Octets:                       0 
		# Rx Broad- and Multicast          0 Tx Broad- and Multicast          0 
		# Rx Error Packets                 0 Tx Error Packets                 0 
		
		%poll_ports_cache = ();
		my $poll_port;
		foreach (split(/\n/, $data))
		{
			if (/^Port\s+(\d+):/)
			{
				$poll_port = $poll_ports_cache{$1} = {};
				next;
			}
			next unless ($poll_port);
			
			if (/^Rx Packets:?\s*(\d+)\s+Tx Packets:?\s*(\d+) /i)
			{
				$poll_port->{'rxGood'} = $1;
				$poll_port->{'txGood'} = $2;
			}
			elsif (/^Rx Octets:?\s*(\d+)\s+Tx Octets:?\s*(\d+) /i)
			{
				$poll_port->{'goodOctetRCV'} = $1;
				$poll_port->{'goodOctetSND'} = $2;
			}
			elsif (/^Rx Broad- and Multicast:?\s*(\d+)\s+Tx Broad- and Multicast:?\s*(\d+) /i)
			{
				$poll_port->{'rxBroadcast'} = $1;
				$poll_port->{'txBroadcast'} = $2;
			}
			elsif (/^Rx Error Packets:?\s*(\d+)\s+Tx Error Packets:?\s*(\d+) /i)
			{
				$poll_port->{'rxFCS'} = $1;
				$poll_port->{'txDrop'} = $2;
			}
		}
	}
	
	##
	## Parses the data, is stored in javascript-variables in the beginning of the page
	##
	
	return $poll_ports_cache{$portnum} || {};
}

##
## Resets stats for all ports
## Not implemented for the 2748, seems to be too much headache
##

sub switch_clear_stat
{
	my $logtag = "$modname: clear_stat:";
	
	#my $res = $ua->get("http://$SET->{'SWITCH'}/stat?page=Overview&submit=Clear");
	#&log_n_die("$logtag Unable to fetch /stat?page=Overview&submit=Clear - ".$res->status_line) unless ($res->is_success);
	&log("Sorry, don't know yet how to clear stat :-(", 1);

	return 1;		# 302 = Probably OK
}

##
## Reading systeminfo that doesn't change (often)
##

sub switch_system_info
{
	my $logtag = "$modname: system_info:";

	my $res = $ua->get("http://$SET->{'SWITCH'}/support/loadsupport.html?R10=0&submit=APPLY", 'Accept-Encoding' => $accept_encoding);
	&log_n_die("Unable to fetch /support/loadsupport.html - ".$res->status_line) unless ($res->is_success);
	
	my $data = $res->decoded_content;
	my $title = '';
	$title = $1 if ($data =~ /<title>([^<]+)<\/title>/);
	
	if ($data =~ /Forbidden Access/ || $title =~ /(Duplicate Administrator|Login)/)
	{
		$SYSTEM->{'cookie'} = '';
		return 0;
	}
	
	&log("Fetched /support/loadsupport.html, got title $title", $SET->{'VERBOSE'} >= 4);
	
	##
	## Finds the info we needs
	##
	
	my $row = 0;
	my %vars = ();
	$title = '';
	
	foreach (split(/\n+/, $data))
	{
		$row++;
		if (/^([^\s].+\:)\s*$/)
		{
			#System Configuration: 
			$title = $1;
			next;
		}
		next unless ($title);
		
		#    Name: PROCURVE J9028B
		next unless (/^\s+([^:]+):(.+)$/);
		
		# System Configuration:Name = PROCURVE J9028B
		$vars{$title.$1} = $2;
	}
	
	##
	## For compability with the Dell output
	##
	
	$SYSTEM->{'model'}      = $vars{'System Configuration:Name'};
	$SYSTEM->{'model'}      =~ s/&nbsp;/ /g;
	
	$SYSTEM->{'macaddress'} = $vars{'System Configuration:MAC address'};
	$SYSTEM->{'macaddress'} =~ tr/-/:/;
	
	$SYSTEM->{'servicetag'} = '';
	$SYSTEM->{'assettag'}   = '';
	$SYSTEM->{'sysname'}    = '';
	$SYSTEM->{'location'}   = '';
	$SYSTEM->{'serial'}     = '';
	
	# PB.03.02
	$SYSTEM->{'firmware'}   = $vars{'System Configuration:S/W Version'} .'/'. $vars{'System Configuration:H/W Version'};
	
	$SYSTEM->{'timeout'}	= 0;
	
	# Not available?
	$SYSTEM->{'uptime'} = 0;
	
	$SYSTEM->{'numberOfPorts'} = 0;
	$SYSTEM->{'numberOfTrunks'} = 0;
	
	my %ports = ();
	
	foreach my $portnum (1 .. 60)
	{
		my $row = $vars{"Port Configuration:$portnum"};
		last unless ($row);
		
		#State     Mode   Flow control  Link    MaxFrame
	    #enabled     auto    disabled      down      1518
		$SYSTEM->{'numberOfPorts'}++;
		my ($state, $mode, $fc, $link, $maxframe) = split(/\s+/, lc $row);
		
		$link =~ s/fdx/\/Full/;
		$link =~ s/hdx/\/Half/;
		
		$ports{$portnum} = {};
		$ports{$portnum}->{'link'} = $link;
		$ports{$portnum}->{'is_active'} = ($link ne 'down');
		$ports{$portnum}->{'order'} = $portnum;
	}
	
	return \%ports;
}

##
## Poll MACs
##

sub switch_poll_macs
{
	my $logtag = "$modname: poll_macs:";
	
	return 0;
}	

##
## Login to setup the session
##

sub switch_login
{
	my $logtag = "$modname: login:";
	
	&log("Logging in into $SET->{'SWITCH'}...", $SET->{'VERBOSE'});
	
	my $res = $ua->get("http://$SET->{'SWITCH'}/login.html?password=$SET->{'PASSWORD'}&submit=Login", 'Accept-Encoding' => $accept_encoding);
	&log_n_die("$logtag Unable to fetch /login.html - ".$res->status_line) unless ($res->is_success);
	
	my $data = $res->decoded_content;
	my $title = '';
	$title = $1 if ($data =~ /<title>([^<]*)<\/title>/);
	print "$data\n";
	
	if ($data =~ /showFailedLogin = "true"/ || $title =~ /(Duplicate Administrator|Login)/)
	{
		if ($title =~ /Duplicate/)
		{
			# Sleeps until the other sessions times out, hopefully
			my $timeout = 10;	#int(($system{'timeout'} || 300) / 2);
			&log("Another session is active, sleeps $timeout secs...", $SET->{'VERBOSE'});
			sleep $timeout;
			
			# Tries again 
			return &switch_login;
		}
		
		&log_n_die("$logtag Unable to login, got '$title' - ".$res->status_line) unless ($res->is_success);
	}
	
	# No cookie, the switch tracks ip
	return 'logged_in';
}

##
## Logout - Not implemented/needed
##

sub switch_logout
{
	my $logtag = "$modname: logout:";
	
	my $res = $ua->get("http://$SET->{'SWITCH'}/logout.html?tmp=0");
	&log_n_die("$logtag Unable to fetch /logout.html - ".$res->status_line) unless ($res->is_success);
	
	my $data = $res->content;
	my $title = '';
	$title = $1 if ($data =~ /<title>([^<]*)<\/title>/);
	print "$data\n";
	
	$SYSTEM->{'cookie'} = '';
	&log("Logged out, got title $title", $SET->{'VERBOSE'} >= 4);
	
	return 1;
}

###########################################################

&log("$modname $modver loaded", $SET->{'VERBOSE'});

1;

