##
## Module to fetch_stat.pl from the Netgear GS108T
##
## Based on work by Joakim Andersson, joakim at iqcc dot se
## Implemented by Ryan Mott, ryan at sproutsoft dot com
##

use strict;
use LWP::UserAgent;
use Net::HTTP;
use URI::Escape;
use Time::HiRes;

our $modver  = '1.00 (2010-09-28)';
our $modname = "netgear_gs108t.pm";
our $modcreator = "Ryan Mott";

#######################################################
# Shared variables

our $SET;			# Settings
our $SYSTEM;		# System-info

# Creating UserAgent-object
my $ua = LWP::UserAgent->new();
$ua->agent("Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");

##
## Poll port
##

sub switch_poll_port
{
	my $portnum = shift;
	my $is_first_poll = shift || 0;
	
	my $logtag = "$modname: poll_port:";
	
	##
	## Is it a port or LAG?
	##
	
	my $arg = ($portnum =~ /^T(\d+)$/) ? "TrunkNo=$1" : "port=$portnum";
		
	# Fetches the page
	my $res = $ua->get("http://$SET->{'SWITCH'}/counter.cgi?$arg&clear=0",
            'Referer' => "http://$SET->{'SWITCH'}/portStatistics.html",
	       'Cookie' => $SYSTEM->{'cookie'}
		 );
	
	&log_n_die("$logtag Unable to fetch /counter.cgi?$arg&clear=0 - ".$res->status_line) unless ($res->is_success);
	
	# Parsing content
	my $data = $res->content;
	$data =~ tr/\r//d;
	
	if ($data =~ /BODY\s+onload=load_login\(\)/)
	{
		##
		## Invalid session!
		##
		
		&log("Session invalid", $SET->{'VERBOSE'});
		
		$SYSTEM->{'cookie'} = '';
		return 0;
	}
	
	##
	## Parses the data, is stored in javascript-variables in the beginning of the page
	##
	
	my $row = 0;
	
	my $name = '';
	my %vars = ();
	
	(   $vars{'goodOctetSND'},  $vars{'goodOctetRCV'}, 
        $vars{'txGood'},        $vars{'rxGood'}, 
        $vars{'txMulticast'},   $vars{'rxMulticast'}, 
        $vars{'txBroadcast'},   $vars{'rxBroadcast'}, 
        $vars{'txDrop'},        $vars{'rxDrop'}, 
        $vars{'txError'},       $vars{'rxError'}, 
        $vars{'txCollisions'},  $vars{'rxUnderSize'}, 
                                $vars{'rxOverSize'}, 
                                $vars{'rxFragment'}, 
                                $vars{'rxJabber'}, 
                                $vars{'rxFCS'}, 
                                $vars{'rx64Octets'}, 
                                $vars{'rx65TO127Octets'}, 
                                $vars{'rx128TO255Octets'}, 
                                $vars{'rx256TO511Octets'}, 
                                $vars{'rx512TO1023Octets'}, 
                                $vars{'rx1024ToMa'}          ) = $data =~ m/^\s*(\d+)$/mg;
		
	return \%vars;
}

##
## Resets stats for all ports
##

sub switch_clear_stat
{
	if ($SYSTEM->{'numberOfPorts'} > 0)
	{
		my $ok = 0;
		foreach my $num (1 .. $SYSTEM->{'numberOfPorts'})
		{
			my $sock = Net::HTTP->new(Host => $SET->{'SWITCH'}) || die $@;
			$sock->write_request(GET => "/counter.cgi?port=$num&clear=1",
				'User-Agent' => "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)",
				'Cache-Control' => 'no-cache',
				'Accept' => '*/*',
				'Cookie' => $SYSTEM->{'cookie'},
				'Referer' => "http://$SET->{'SWITCH'}/counter.cgi",
				);
			
			my ($code, $mess, %h) = $sock->read_response_headers;
			$ok += ($code == 200);
		}
		print "Cleared $ok ports\n";
	}
	if ($SYSTEM->{'numberOfTrunks'} > 0)
	{
		my $ok = 0;
		print "Cleared $ok LAGs\n";
	}
	
	return 1;
}

##
## Reading systeminfo that doesn't change (often)
##

sub switch_system_info
{
	my $logtag = "$modname: system_info:";
	
	my $res = $ua->get("http://$SET->{'SWITCH'}/UseCookies.js",
	       'Cookie' => $SYSTEM->{'cookie'}
		 );
	
	&log_n_die("$logtag Unable to fetch /UseCookies.js - ".$res->status_line) unless ($res->is_success);
	
	# Content
	my $data = $res->content;
	$data =~ tr/\r//d;
	
	if ($data =~ /BODY\s+onload=load_login\(\)/)
	{
		##
		## Invalid session!
		##
		
		&log("Session invalid", $SET->{'VERBOSE'});
		
		$SYSTEM->{'cookie'} = '';
		return 0;
	}
	
	##
	## Finds the info we needs
	##
	
	my $row = 0;
	my %vars = ();
	foreach (split(/\n+/, $data))
	{
		$row++;
		#print "RAD $row: $_\n";
		
		last if (/^function/);
		next unless (/^var\s*([^\s=]*)\s*=\s*"?(\d*)"?;$/);
		
		my $name = $1;
		my $value = $2;
		
		#next if ($stat{$name});
		#print "  $name -> $value\n";
		$vars{$name} = $value;
	}
	
	$SYSTEM->{'numberOfPorts'} = $vars{'port_num'};
	$SYSTEM->{'numberOfTrunks'} = 0; #$vars{'multi_num'};
	
	##
	## Fetching port info
	##
	
	$res = $ua->get("http://$SET->{'SWITCH'}/portsConfiguration.html",
	       'Cookie' => $SYSTEM->{'cookie'}
		 );
	
	&log_n_die("$logtag: Unable to fetch /portsConfiguration.html - ".$res->status_line) unless ($res->is_success);
	
	# Parsing content
	$data = $res->content;
	$data =~ tr/\r//d;
	
	if ($data =~ /BODY\s+onload=load_login\(\)/)
	{
		##
		## Invalid session!
		##
		
		&log("Session invalid", $SET->{'VERBOSE'});
		
		$SYSTEM->{'cookie'} = '';
		return 0;
	}

	##
	## Parses info about every port
	##
	
#	my $tot = $SYSTEM->{'numberOfPorts'} + $SYSTEM->{'numberOfTrunks'};
	my %ports = ();
	my $portnum = 1;

	while ($data =~ m/^\s*st\[i\]\s*=\s*"([^.]*)\.([^.]*)\.([^.]*)\.([^.]*)\.([^.]*)\.([^"]*)/mg) {
	    my $portLink   = $2 + 0;
        my $portSpeed  = $3 + 0;
        my $portDuplex = $4 + 0;
        my $portFc     = $5 + 0;

        #my $portBP     = 0; #
		my $link = 'down';
		
		if ($portLink > 0)
		{
			$link = ('1000','100','10','unknown')[$portSpeed];
			$link .= ('/Full','/Half','')[$portDuplex];
			$link .= '/FC' if ($portFc == 1); # flow control
			#$link .= '/BP' if ($portBP == 1); # backpressure
		}
		
		$ports{$portnum} = {};
		$ports{$portnum}->{'link'} = $link;
		$ports{$portnum}->{'is_active'} = $portLink;
		$ports{$portnum}->{'order'} = $portnum++;
	}

	##
	## Fetching system info
	##
	
	$res = $ua->get("http://$SET->{'SWITCH'}/sysInfo.html",
	       'Cookie' => $SYSTEM->{'cookie'}
		 );
	
	&log_n_die("$logtag: Unable to fetch /sysInfo.html - ".$res->status_line) unless ($res->is_success);
	
	# Parsing content
	$data = $res->content;
	$data =~ tr/\r//d;
	
	if ($data =~ /BODY\s+onload=load_login\(\)/)
	{
		##
		## Invalid session!
		##
		
		&log("Session invalid", $SET->{'VERBOSE'});
		
		$SYSTEM->{'cookie'} = '';
		return 0;
	}
	
	##
	## Parses the data

	%vars = ();
	( $vars{'modelName'}, $vars{'hardwareVersion'}, $vars{'bootVersion'}, $vars{'softwareVersion'} ) = $data =~ m!(?:e|r)" align="center">([^<]*)</td>!smg;
	( $vars{'systemName'}, $vars{'locationName'} ) = $data =~ m/inputBox_[^\s=\\.+]*[\s="]+([^"]*)/smg;
	( $vars{'macAddr'} ) = $data =~ m!>([-:0-9a-fA-F]{12,17})(?=</td>)!smg;
	( $vars{'sysUpTime'} ) = $data =~ m!System UpTime.+?>([^<]+)</td!smg;

	my $secs = 0;
    if ($data =~ m!p">(?>(\d+) years)?\s?(?>(\d+) days)?\s?(?>(\d+) hours)?\s?(?>(\d+) mins)?\s?(?>(\d+) seconds)?</td!sm) {
        #my $years = $1 + 0;
        my $days = $2 + 0;
        my $hours = $3 + 0;
        my $mins = $4 + 0;
        my $seconds = $5 + 0;
        $secs = $seconds + $mins*60 + $hours*3600 + $days*86400; # + $years*31536000;
    }
    
	# Populating SYSTEM with the vars
	$SYSTEM->{'model'}      = $vars{'modelName'};
	$SYSTEM->{'sysname'}    = $vars{'systemName'};
	$SYSTEM->{'location'}   = $vars{'locationName'};
	$SYSTEM->{'macaddress'} = $vars{'macAddr'};
	$SYSTEM->{'servicetag'} = $SYSTEM->{'assettag'} = $SYSTEM->{'serial'} = "N/A";
	$SYSTEM->{'firmware'}   = "$vars{'softwareVersion'}/$vars{'bootVersion'} hw $vars{'hardwareVersion'}";
	
	$SYSTEM->{'uptime'} = $secs;
	
	return \%ports;
}

##
## Poll MACs
##

sub switch_poll_macs
{
	my $logtag = "$modname: poll_macs:";
	
	return 0;
}	

##
## Login to setup the session
##

sub switch_login
{
	my $logtag = "$modname: login:";
	
	my $urlenc_password = uri_escape($SET->{'PASSWORD'});
	my $rtime = Time::HiRes::time * 2;
	
    my @headers = ();
	my $sock = Net::HTTP->new(Host => $SET->{'SWITCH'}) || die $@;
	$sock->write_request(POST => "/login.cgi",
					'User-Agent' => "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)",
					'Content-Type' => 'application/x-www-form-urlencoded',
					'Cache-Control' => 'max-age=0',
					'Accept' => '*/*',
					"password=$urlenc_password&rtime=$rtime"
			);

	my ($code, $mess, @headers) = $sock->read_response_headers();

	unless ($code == 302)
	{
		&log_n_die("$logtag Didn't recieve code 302 when logging in! $mess");
	}
	
	##
	## Finding the Cookie we should recieve when logged in successfully
	##
	## Set-Cookie: Broadcom-WebSuperSmart=0BF4662C006F2E45006F2E45; path=/
	##
	
	my @hmatch = grep(/Broadcom-WebSuperSmart=/, @headers);
	unless (@hmatch[0])
	{
		foreach (@headers)
		{
			print "$_\n";
		}
			
		&log_n_die("$logtag Didn't recieve cookie! @hmatch[0]");
	}
	
	@hmatch[0] =~ /(Broadcom-WebSuperSmart=.+?)\;/ || &log_n_die("$logtag Unable to interpret 'Broadcom-WebSuperSmart' cookie from @hmatch[0]!");
	return $1;
}

##
## Logout
##

sub switch_logout
{
	my $logtag = "$modname: logout";
	
	# there is no logout page for this device, just drop the cookie
	
	# Assume it worked
	$SYSTEM->{'cookie'} = '';
	
	return 1;
}

###########################################################

&log("$modname $modver loaded", $SET->{'VERBOSE'});

1;

